//=========================================================
// src/Interrupts.c: generated by Hardware Configurator
//
// This file will be regenerated when saving a document.
// leave the sections inside the "$[...]" comment tags alone
// or they will be overwritten!
//=========================================================


// USER INCLUDES
#include <SI_EFM8UB1_Register_Enums.h>
#include "AmbientLightMonitor_Defs.h"

//global variables
unsigned char SPITxCount;
unsigned char LCDLineAddress;
unsigned char LCDLineStart;
unsigned char LCDLineStop;
unsigned char LCDLineDataIndex;
bit ADC_CONV_COMPLETE = FALSE;

//-----------------------------------------------------------------------------
// SPI0_ISR
//-----------------------------------------------------------------------------
//
// SPI0 ISR Content goes here. Remember to clear flag bits:
// SPI0CN0::MODF (Mode Fault Flag)
// SPI0CN0::RXOVRN (Receive Overrun Flag)
// SPI0CN0::SPIF (SPI# Interrupt Flag)
// SPI0CN0::WCOL (Write Collision Flag)
// SPI0FCN1::RFRQ (Receive FIFO Request)
// SPI0FCN1::TFRQ (Transmit FIFO Request)
//
//-----------------------------------------------------------------------------
SI_INTERRUPT (SPI0_ISR, SPI0_IRQn)
{
	SFRPAGE = SPI0_PAGE;
	SPI0CN0_SPIF = 0;															//clear the interrupt flag

	/*This is the state machine that handles the SPI transfers, implemented with a switch statement.
	 The value of LCDTxState determines which instructions are executed. One switch statement is used
	 for both all clear transfers and line data transfers. A flowchart that clarifies the operation of this state machine
	 is provided in the accompanying article */
	switch(LCDTxState)
	{
		case ALL_CLEAR_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = 0x00;
			LCDTxState = FIRST_ALLCLEAR_DUMMY_SENT;
			break;

		case FIRST_ALLCLEAR_DUMMY_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = 0x00;
			LCDTxState = SECOND_ALLCLEAR_DUMMY_SENT;
			break;

		case SECOND_ALLCLEAR_DUMMY_SENT:
			SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 4);			//this provides the required delay between the last clock cycle and chip select going low (thSCS in the LCD data sheet)
			SPI_CS = LOW;																							//set chip select LOW to complete the transfer
			SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 12);			//this ensures adequate delay between chip select going low then high again (twSCSL in the LCD data sheet)
			LCDTxState = IDLE;
			break;

		case DATA_UPDATE_MODE_SENT:
			LCDLineAddress = LCDLineStart;
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = LCDLineAddr_LUT[LCDLineAddress];
			LCDTxState = FIRST_LINE_ADDR_SENT;
			LCDLineDataIndex = 0;
			break;

		case FIRST_LINE_ADDR_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = LCDDisplayData[LCDLineAddress - DIGIT_BEGIN_LINE][LCDLineDataIndex];
			LCDLineDataIndex++;
			if(LCDLineDataIndex == NUM_LINE_DATA_BYTES)
				{LCDTxState = LINE_DATA_SENT;
				LCDLineAddress++;
				LCDLineDataIndex = 0;}
			break;

		case LINE_DATA_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = 0x00;
			LCDTxState = PRE_ADDR_DUMMY_SENT;
			break;

		case PRE_ADDR_DUMMY_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = LCDLineAddr_LUT[LCDLineAddress];
			LCDTxState = NEW_ADDR_SENT;
			break;

		case NEW_ADDR_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = LCDDisplayData[LCDLineAddress - DIGIT_BEGIN_LINE][LCDLineDataIndex];
			LCDLineDataIndex++;
			if(LCDLineDataIndex == NUM_LINE_DATA_BYTES)
			{
				LCDTxState = LINE_DATA_SENT;
				LCDLineAddress++;
				LCDLineDataIndex = 0;
				if(LCDLineAddress == LCDLineStop)
					LCDTxState = LAST_LINE_DATA_SENT;
			}
			break;

		case LAST_LINE_DATA_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = 0x00;
			LCDTxState = FIRST_POST_DATA_DUMMY_SENT;
			break;

		case FIRST_POST_DATA_DUMMY_SENT:
			SFRPAGE = SPI0_PAGE;
			SPI0DAT = 0x00;
			LCDTxState = SECOND_POST_DATA_DUMMY_SENT;
			break;

		case SECOND_POST_DATA_DUMMY_SENT:
			SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 4); 		//this provides the required delay between the last clock cycle and chip select going low (thSCS in the LCD data sheet)
			SPI_CS = LOW;																							//set chip select LOW to complete the transfer
			SFRPAGE = TIMER4_PAGE; TMR4L = 0; while(TMR4L < 12);			//this ensures adequate delay between chip select going low then high again (twSCSL in the LCD data sheet)
			LCDTxState = IDLE;
			break;
	}

}


//-----------------------------------------------------------------------------
// ADC0EOC_ISR
//-----------------------------------------------------------------------------
//
// ADC0EOC ISR Content goes here. Remember to clear flag bits:
// ADC0CN0::ADINT (Conversion Complete Interrupt Flag)
//
//-----------------------------------------------------------------------------
SI_INTERRUPT (ADC0EOC_ISR, ADC0EOC_IRQn)
{
	SFRPAGE = ADC0_PAGE;
	ADC0CN0_ADINT = 0;					//clear the interrupt flag

	ADC_CONV_COMPLETE = TRUE;
}


